import requests


__all__ = ["SberbankAPI"]


class SberbankAPI:
    """
    Класс для работы с api платежного шлюза Сбербанка

    Включает в себя методы:
        1. register_order - Регистрация зазказа
        2. get_status_order - Получение статуса об оплате
    """

    def __init__(self, username, password, api_url, redirect_success, redirect_fail):
        """
        Инициализация класса

        :param username: Имя пользователя для интеграции по API
        :param password: Пароль пользователя для интеграции по API
        :param api_url: URL адрес, REST интерфейса API Сбербанка
        :param redirect_success: URL адрес, для перенаправления после успешной оплаты
        :param redirect_fail: URL адрес, для перенаправления после неудачной оплаты
        """
        self.username = username
        self.password = password
        self.main_url = api_url
        self.redirect_success = redirect_success
        self.redirect_fail = redirect_fail

    async def __send_request__(self, method: str, data: dict) -> dict:
        """
        Отправка запроса в платежный шлюз Сбербанка

        :param method: Метод
        :param data: Параметры запроса
        :return: Ответ от платежного шлюза Сбербанка в формате json
        """
        headers = {"Content-type": "application/x-www-form-urlencoded"}
        params = {
            "userName": self.username,
            "password": self.password,
        }
        params.update(data)
        r = requests.post(url=f'{self.main_url}{method}', headers=headers, params=params)
        return r.json()

    async def register_order(
            self,
            order_number: str,
            amount: int,
            return_url: str = None,
            fail_url: str = None
    ) -> dict:
        """
        Метод регистрации заказа в платежном шлюзе Сбербанка

        :param order_number: Уникальный идентификатор заказа в пределах магазина
        :param amount: Общая сумма заказа в минимальных единицах валюты
        :param return_url: URL адрес, на который требуется перенаправить пользователя в случае успешной оплаты.
        :param fail_url: URL адрес, на который требуется перенаправить пользователя в случае неуспешной оплаты.
        :return: Рузультат выполнения функции __send_request__
        """
        return_url = return_url if return_url else self.redirect_success
        fail_url = fail_url if fail_url else self.redirect_fail

        params = {"orderNumber": order_number, "amount": amount, "returnUrl": return_url, "failURL": fail_url}
        return await self.__send_request__("register.do", params)

    async def get_status_order(self, order_id: str, shop_order_id: str) -> dict:
        """
        Получение информации о заказе

        :param order_id: Идентификатор заказа в платежном шлюзе сбербанка
        :param shop_order_id: Идентификатор заказа в магазине
        :return: Рузультат выполнения функции __send_request__
        """
        params = {"orderId": order_id, "orderNumber": shop_order_id}
        return await self.__send_request__("getOrderStatusExtended.do", params)



from functools import wraps

from flask import _request_ctx_stack, has_request_context, session


class UserSession:
    User = None

    def __init__(self, user_model, type_db: str, dev_mode: bool = False):
        """
        :param user_model Модель пользователя, для аунтификации
        :param type_db Тип базы данных (sql/nosql)
        :param dev_mode Режим разработки. Флаг True/False
        """
        self.User = user_model
        self.type_db = type_db
        self.dev = dev_mode

    @staticmethod
    def authenticate(user_instance=None, email: str = None, password: str = None, token: str = None):
        """
        Аутентифицирует пользователя по паре email-password

        :param user_instance: Пользователь
        :param email: Email пользователя
        :param password: Пароль пользователя в открытом виде
        :param token: Токен пользователя
        :return: Аутентифицированный пользователь в случае совпадения пароля, иначе None
        """
        try:
            if not user_instance:
                user_instance = user_instance.get_by_email(email, status='active')
            if not user_instance:
                return None

            if password:
                assert user_instance.check_password(password)
                return user_instance

            if token:
                assert user_instance.check_token(token)
                return user_instance
        except AssertionError:
            pass
        return None

    @staticmethod
    def login(user_instance, remember: bool = False) -> bool:
        """
        Аутентифицирует пользователя в текущей сессии

        :param user_instance: Пользователь, для которого открывается сессия
        :param remember: Флаг запоминания пользователя после окончания сессии
        :return: Результат открытия аутентифицированной сессии
        """
        if not user_instance.active:
            return False

        user_id = user_instance.id
        session['user_id'] = str(user_id)
        if hasattr(_request_ctx_stack.top, 'user'):
            delattr(_request_ctx_stack.top, 'user')

        if remember:
            session['remember'] = remember
        return True

    @staticmethod
    def logout():
        """
        Заканчивает активную пользовательскую сессию

        :return: Результат завершения сессии
        """
        session.pop('user_id', None)
        _request_ctx_stack.top.user = None
        return True

    def get_current_user(self):
        """Получает текущего пользователя"""
        if has_request_context() and not hasattr(_request_ctx_stack.top, 'user'):
            if user_id := session.get('user_id'):
                if self.type_db == 'nosql':
                    user = self.User.objects.filter(state='active', id=user_id).first()
                    _request_ctx_stack.top.user = user
                elif self.type_db == 'sql':
                    user = self.User.where(state='active', id=user_id).first()
                    _request_ctx_stack.top.user = user
            elif self.dev and hasattr(_request_ctx_stack.top.request.authorization, 'username') \
                    and hasattr(_request_ctx_stack.top.request.authorization, 'password'):
                email = _request_ctx_stack.top.request.authorization.username
                token = _request_ctx_stack.top.request.authorization.password

                if (user := self.User.get_by_email(email)) and user.check_token(token):
                    _request_ctx_stack.top.user = user

        return getattr(_request_ctx_stack.top, 'user', None)

    def login_required(self, local_proxy: bool = False):
        """
        Декторатор требующий обязательной авторизации

        :param local_proxy
        :return Результат выполнения декорируемой функции
        """

        def decorator(func):
            @wraps(func)
            def wrapper(*args, **kwargs):
                if not self.get_current_user():
                    return {'errors': {"auth": 'Not authenticated'}}, 401
                return func(*args, **kwargs)
            return wrapper
        return decorator

    def need_role(self, role_keys: list, field='role'):
        """
        Декоратор, допускает выполнение метода для пользователя с определенной ролью

        :param role_keys Список допустимых ролей
        :param field Поле в модели, в которой указана роль. (По умолчанию - role)
        :return Результат выполнения декорируемой функции
        """
        def decorator(func):
            @wraps(func)
            def wrapper(*args, **kwargs):
                current_user = self.get_current_user()
                if not current_user:
                    return {"errors": {"auth": "No authentication provided"}}, 401
                if getattr(current_user, field) not in role_keys:
                    return {"errors": {"role": "insufficient rights for {} role".format(current_user.role)}}, 403
                return func(*args, **kwargs)
            return wrapper
        return decorator



import time
from typing import Any


def check_run_time(count_decimal_places: int = 2) -> Any:
    """
    Проверка времени выполнения функции

    :param count_decimal_places Количество знаков после запятой
    """
    def decorator(func):
        def wrapper(*args, **kwargs):
            start_time = time.time()
            func(*args, **kwargs)
            finish_time = time.time()
            return finish_time-start_time
        return wrapper
    return decorator




import json
import random
import string
import unittest
from os import getcwd
from os.path import dirname
from typing import Any

from marshmallow import Schema


class CommonTestCase(unittest.TestCase):
    test_db_name = None
    db = None
    client = None
    app_context = None
    maxDiff = None
    authorized = False
    url = None
    request_method = None
    user_model = None

    @classmethod
    def setUpClass(cls, *args):
        """
        Start Flask app end check test database name in current db

        Please pass three arguments:
            args[0] - Function for create flask app
            args[1] - Test config for flask app
            args[2] - DB
        """
        if len(args) < 3:
            raise AssertionError('Please pass three arguments')
        create_flask_func = args[0]
        config = args[1]
        cls.db = args[2]

        # Start flask app
        app = create_flask_func(config)
        cls.client = app.test_client()
        cls.app_context = app.app_context()
        cls.app_context.push()

        # Check test database name in current db
        cls.test_db_name = app.config['TEST_DB_NAME']
        dev_db_name = app.config['DEV_DB_NAME']
        if cls.test_db_name in cls.db.connection.database_names():
            cls.db.connection.drop_database(cls.test_db_name)

        # Create all collections from dev db
        for collection_name in cls.db.connection[dev_db_name].list_collection_names():
            cls.db.connection[cls.test_db_name].create_collection(name=collection_name)

    @classmethod
    def tearDownClass(cls):
        """Delete test data and stop Flask app"""
        # Удаление тестовой базы и завершение Flask приложения
        cls.db.connection.drop_database(cls.test_db_name)
        cls.app_context.pop()

    @classmethod
    def create_user(cls, username='unit@test.ru', password='test_pass', first_name='test', **other_data):
        user_data = {"email": username, "first_name": first_name, "phone": '+1234567890', **other_data}
        if not (auth_user := cls.user_model.objects(**user_data).first()):
            auth_user = cls.user_model.objects.create(**user_data)
        auth_user.set_password(password)
        auth_user.save()
        return auth_user

    def auth(self, auth_url: str = '/api/login/', username: str = 'unit@test.ru', password: str = 'test_pass'):
        """
        Authorization function.

        :param auth_url URL for authorization
        :param username Username
        :param password Password
        """
        json_request = {"email": username, "password": password}
        response = self.client.post(auth_url, json=json_request)
        self.authorized = response.status_code == 200

    def validate_invalid_doc_id(self, error_type='bad_id', error_message_substr='',
                                id_in='url', _id=None, data=None, doc_field='id', status_code=400):
        """
        Validate invalid identifier

        :param error_type
        :param error_message_substr
        :param id_in
        :param _id
        :param data
        :param doc_field
        :param status_code
        """

        args = (self.url,)
        params = {}
        if id_in == 'data':
            params['json'] = {doc_field: _id} if data is None else data
        response = self.request_method(*args, **params)
        json_response = self.check_response(response, status_code)
        self.assertIn('errors', json_response)
        self.assertIn(doc_field, json_response['errors'])
        if error_type == 'not_found':
            self.assertIn('Could not find document.', json_response['errors'][doc_field])
        elif error_type == 'bad_id':
            self.assertEqual(json_response['errors'][doc_field], ['Invalid identifier'])
        if error_message_substr:
            self.assertIn(error_message_substr, json_response['errors'][doc_field])

    def validate_forbidden_access(self, role_keys: list):
        """
        Validate forbidden access

        :param role_keys List not allowed roles
        """
        for role in role_keys:
            self.client.cookie_jar.clear()
            user = self.create_user(username=f'{role}@forbidden.com', password='pass', role=role)
            self.auth(username=user.email, password='pass')
            json_response = self._send_request(expected_status_code=403)
            self.assertIn('errors', json_response)
            self.assertIn("role", json_response['errors'])
            self.assertEqual(f"insufficient rights for {role} role", json_response['errors']['role'])

    def validate_field_in_bad_request(self, field_name: str, valid_type: Any, field_is_required: bool = False):
        """
        Success validate field in bad request

        :param field_name Field name
        :param valid_type Valid type for this field
        :param field_is_required Field is required in request? True/False
        """
        data = {}
        for invalid_param in self.generate_bad_data(valid_type=valid_type):
            data[field_name] = invalid_param
            json_response = self._send_request(params=data, expected_status_code=400)
            self.assertIn('errors', json_response)
            self.assertIn(field_name, json_response['errors'])
        if field_is_required:
            self.validate_required_field(field_name)

    def validate_required_field(self, field_name: str):
        """
        Validate required field

        :param field_name Field is required in request
        """
        json_response = self._send_request(params={}, expected_status_code=400)
        self.assertIn('errors', json_response)
        self.assertIn(field_name, json_response['errors'])

    def validate_error_parse_json(self):
        """Check request. Error, if not json in request"""
        json_response = self._send_request(expected_status_code=400)
        self.assertIn('errors', json_response)
        self.assertIn('common', json_response['errors'])
        self.assertIn('Cannot parse json', json_response['errors']['common'])

    def validate_json(self, response_json, schema):
        """Validate json response"""
        self.assertIsNotNone(response_json)
        validation_errors = schema(unknown='exclude').validate(response_json)
        if validation_errors:
            print(f"Ошибки при валидации ответа: \n{validation_errors}")
        self.assertDictEqual(validation_errors, {})

    def validate_limit(self, return_schema: Schema, limit: int = 20):
        """
        Validate response and limit from GET method

        :param return_schema Marshmallow Schema for validate response
        :param limit Check limit
        """
        json_response = self._send_request()
        self.validate_json(json_response, return_schema)
        self.assertEqual(len(json_response['items']), limit)

    def validate_offset(self, return_schema):
        """
        Validate offset. GET Method

        :param return_schema Marshmallow Schema for validate response
        """
        json_response = self._send_request(params={'limit': 2})
        self.validate_json(json_response, return_schema)
        total_count = json_response['total_count']

        # Set second identifier to var
        self.assertEqual(len(json_response['items']), 2)
        second_doc_id = json_response['items'][1]['id']

        # Request offset=1&limit=1, the identifier specified in second_doc_id is expected
        json_response = self._send_request(params={'limit': 1, 'offset': 1})
        self.validate_json(json_response, return_schema)
        self.assertEqual(json_response['total_count'], total_count)
        self.assertEqual(len(json_response['items']), 1)
        self.assertEqual(json_response['items'][0]['id'], second_doc_id)

    def validate_filter(self,
                        return_schema: Schema,
                        field: str,
                        value: str,
                        check_value: bool = True,
                        icontains: bool = False):
        """
        Validate filtered response

        :param return_schema Marshmallow Schema for validate response
        :param field Filter by field
        :param value value filter
        :param check_value Check value in response
        :param icontains True/False
        """
        json_response = self._send_request(params={field: value})
        self.validate_json(json_response, return_schema)
        items = json_response['items']
        if check_value:
            for item in items:
                self.assertIn(value, item[field]) if icontains else self.assertEqual(value, item[field])
        return items

    def create_success(self, model, required_data):
        """Create success. Only required fields"""
        json_response = self._send_request(params=required_data, expected_status_code=201)
        instance = model.objects.filter(pk=json_response['id']).first()
        self.assertNotEqual(instance, None)
        instance.delete()

    def edit_success(self, edit_obj, edit_field: str, new_value: str, check_new_value=True):
        """
        Success edit object.

        :param edit_obj Object for edit
        :param edit_field Edit field
        :param new_value New value
        :param check_new_value Check new value in edit field. True/False
        """
        json_response = self._send_request(params={edit_field: new_value})
        self.assertIn('status', json_response)
        self.assertEqual('success', json_response['status'])
        edit_obj.reload()
        if check_new_value:
            self.assertEqual(getattr(edit_obj, edit_field), new_value)

    def delete_success(self, delete_obj, deleted_state='hidden'):
        """
        Success delete object

        :param delete_obj Object for delete
        :param deleted_state Deleted state. For check deleted doc.
        """
        json_response = self._send_request(params={"id": delete_obj.id})
        self.assertIn('status', json_response)
        self.assertEqual('success', json_response['status'])
        self.assertEqual(getattr(delete_obj, "state"), deleted_state)

    def check_response(self, response, status_code=200):
        self.assertEqual(response.status_code, status_code)
        self.assertTrue(response.is_json)
        try:
            return response.json
        except Exception:
            self.assertTrue(False)
            return None

    @staticmethod
    def generate_test_data(model, key: str, many: bool = False, count: int = 21, other_data: dict = None):
        """
        Generate test data for devices tests. This method reading file ./test_data.json

        :param model Model instance
        :param key Key in data json
        :param many Create many instances. True/False
        :param count Count create instances. Only many=True.
        :param other_data Other data for create or update default data

        """
        other_data = other_data if other_data else {}
        count_create = count if many else 1
        instance = None
        instances = []

        def get_data_from_file():
            """Read data in json file"""
            test_dir = getcwd()
            if test_dir.split('/')[-1] != 'tests':
                test_dir = dirname(test_dir)
            with open(f'{test_dir}/test_data.json', encoding='utf-8') as file:
                return json.load(file).get(key)

        data = get_data_from_file()
        data.update(other_data)
        for i in range(count_create):
            data = {key: f(value, i=i) if isinstance(value, str) else value for key, value in data.items()}
            if not (instance := model.objects(**data).first()):
                instance = model.objects.create(**data)
            instances.append(instance)
        if not many or count_create == 1:
            return instance
        else:
            return instances

    def generate_bad_data(self, valid_type=None, max_length=None, min_length=None):
        self.assertIsNotNone(valid_type)
        invalid_data_map = {
            int: [None, True, "", {}, [], "string", "string1", {"key": "value"}, ["item1"], [1, 2], 1.45],
            float: [None, True, "", {}, [], "string", "string1", {"key": "value"}, ["item1"], [1, 2]],
            str: [None, True, {}, [], 1, {"key": "value"}, ["item1"], [1, 2]],
            bool: [None, "", {}, [], 123, "string", "string1", {"key": "value"}, ["item1"], [1, 2], 1.45],
            list: [None, "", {}, 123, "string", "string1", {"key": "value"}, 1.45],
            "date": [None, True, {}, [], 1, "string", {"key": "value"}, ["item1"], [1, 2], '2020-01-01 10:10'],
            "datetime": [None, True, {}, [], 1, "string", {"key": "value"}, ["item1"], [1, 2], '2020-01-01'],
            "email": [1, None, True, [], {}, "", "string", {"k": "v"}, ["i"], [1], 1.2]
        }
        bad_data = invalid_data_map[valid_type]

        # TODO Сделать более универсальным max_length min_length
        if max_length is not None:
            bad_item = ""
            for item in range(max_length + 1):
                bad_item += "s"
            bad_data.append(bad_item)

        if min_length is not None:
            if valid_type == str:
                bad_item = ''.join(random.choice(string.ascii_letters + string.digits)
                                   for _ in range(1, min_length))
                bad_data.append(bad_item)
            else:
                bad_data.append(0)

        return bad_data

    def _send_request(self,
                      url: str = None,
                      params: dict = None,
                      return_to_json: bool = True,
                      expected_status_code: int = 200):
        """
        Send request method.

        :param url String url for request
        :param params Parameters for request
        :param return_to_json True/False
        :param expected_status_code Allowed status code in response
        :return Response or json_response
        """
        url_for_request = url if url else self.url
        request_params = {"json": params}
        if self.request_method == self.client.get:
            request_params['params'] = request_params.pop('json', {})
        response = self.request_method(url_for_request, **request_params)
        if return_to_json:
            return self.check_response(response, status_code=expected_status_code)
        return response



import requests


class EventMessenger:

    """
    Класс для отправки событий в чат ТГ
    """

    def __init__(self, url: str, secret: str = None, host: str = None, chat_id: str = None):
        """
        :param url Ссылка на облачную функцию
        """
        self.url = url

    def send_message(self, message: str, recipient_id: str):
        """
        Отправка сообщения

        :param recipient_id Идентификатор получателя
        :param message Тело сообщения
        """
        json = {
            'chat_id': recipient_id,
            'message': message
        }
        requests.post(self.url, json=json)



from functools import partial
from typing import Any

from marshmallow import ValidationError
from mongoengine import ValidationError as MongoValidationError


def convert_to_instance(
        model: Any,
        type_db: str,
        field: str = 'id',
        many: bool = False,
        allow_deleted: bool = True,
        check_deleted_by: str = 'state',
        return_field: str = None,
        assert_every=True,
        error: str = 'Could not find document.'):
    """
    Convert to instance

    :param model The model to which you want to convert
    :param type_db Database type (sql/nosql)
    :param field The field that will be converted
    :param many Convert to many instances flag
    :param allow_deleted Allowed return deleted instances flag
    :param check_deleted_by Filed, by check deleted instances. (If allow_deleted=False)
    :param return_field Return value field in current instance
    :param assert_every Check every doc
    :param error Error message, by not found document

    :return Instance or value field in instance or errors

        Example:
            data = {"attribute_name": 123456}
            ...
            class ClassName(Schema):
                attribute_name = fields.Function(deserialize=to_instance(User, 'nosql'))
            ...
            result = ClassName().load(data)     # {"attribute_name": UserInstance}
    """

    def get_value_from_instances(instances, **kwargs):
        """Get field in instances"""
        instances = instances if isinstance(instances, list) else [instances]
        fields = model.columns if kwargs['type_db'] == 'sql' else model._fields.keys()
        if return_field in fields:
            return [getattr(doc, return_field) for doc in instances]
        raise ValidationError("Field not found in model")

    def query(value, many_instances=False, **kwargs):
        """Query for sql/nosql database"""
        # Generate filter data
        query_field = f"{field}__in" if many_instances else field
        filter_data = {query_field: value}
        if not allow_deleted:
            filter_data.update({f'{check_deleted_by}__ne': 'deleted'})

        # Generate query
        if kwargs['type_db'] == 'sql':
            return model.where(**filter_data)
        elif kwargs['type_db'] == 'nosql':
            return model.objects.filter(**{query_field: value})
        else:
            ValidationError(error, field_name=field)

    def convert_one(value, **kwargs):
        """Convert to one instance"""
        return query(value, **kwargs).first()

    def convert_many(value, **kwargs):
        """Convert to many instances"""
        if value.startswith('[') and value.endswith(']'):
            values = value[2:-2].replace("'", "").split(',')
        elif isinstance(value, str):
            values = value.split(',')
        else:
            raise ValidationError(message='Invalid type data', field_name=field)
        values = list(set(values))
        result = query(values, many_instances=True, **kwargs).all()
        if assert_every and len(result) != len(values):
            raise ValidationError(message='Not all documents were found', field_name=field)
        return result

    def to_instance(*args, **kwargs):
        """Main func"""
        value = str(args[0])
        try:
            result = convert_many(value, **kwargs) if many else convert_one(value, **kwargs)
        except MongoValidationError:
            raise ValidationError('Invalid identifier', field_name=field)
        if not result:
            raise ValidationError(error, field_name=field)
        return get_value_from_instances(result, **kwargs) if kwargs.get('return_field') else result

    return partial(to_instance, model=model, type_db=type_db, field=field,
                   many=many, error=error, return_field=return_field)



from functools import wraps

from marshmallow import ValidationError

from flask import request


def get_params(schema, **schema_params):
    """
    Getting request parameters

    :param schema: Marshmallow schema
    :param schema_params: exclude=[], only=[], partial=True/False, unknown='exclude'
    :return: request params
    """

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            data = None
            if request.method == 'GET':
                data = request.args
            elif request.method in ['POST', 'PUT', 'DELETE']:
                if not request.is_json:
                    return {'errors': {"common": "Cannot parse json"}}, 400
                data = request.json

            # Load params
            try:
                params = (schema(**schema_params).load(data),)
            except ValidationError as exc:
                return {'errors': exc.messages}, 400
            args += params
            return func(*args, **kwargs)

        return wrapper

    return decorator


def convert_to_instance(
        model,
        type_db: str,
        field: str = 'id',
        allow_deleted: bool = False,
        check_deleted_by: str = 'state',
        optional_filter: dict = None,
        error: str = 'Could not find document.'):
    """
    Convert to instance decorator

    :param model: Model
    :param type_db: Type database (sql/nosql)
    :param field: Convert to instance by field in model. Equal in url var name in urls.py. /api/examples/<var_name>/
    :param allow_deleted: Allow return deleted instance
    :param check_deleted_by: Check deleted by field
    :param optional_filter Optional filter
    :param error Error message, by not found document
    :return Instance or error
    """
    if optional_filter is None:
        optional_filter = {}

    def to_instance_nosql(filter_data):
        """Convert to instance from nosql db"""
        from mongoengine import ValidationError as MongoValidationError
        try:
            return model.objects.filter(**filter_data).first(), None
        except MongoValidationError:
            return None, {'errors': {field: 'Invalid data'}}

    def to_instance_sql(filter_data):
        """Convert to instance from sql db"""
        return model.where(**filter_data).first(), None

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            """Main func"""
            filter_data = {field: kwargs.pop(field), **optional_filter}
            if not allow_deleted:
                filter_data.update({f'{check_deleted_by}__ne': 'deleted'})
            doc, errors = to_instance_nosql(filter_data) if type_db == 'nosql' else to_instance_sql(filter_data)
            if errors:
                return errors, 400
            if not doc:
                return {'errors': {field: error}}, 400
            args += (doc,)
            return func(*args, **kwargs)
        return wrapper
    return decorator



import requests


class AvitoAPI:

    """Класс для работы с API avito.ru"""

    def __init__(self, client_id, client_secret, user_id=None):
        """
        Инициализация класса

        :param client_id Идентификатор магазина
        :param client_secret Секретный ключ магазина
        """
        self.client_id = client_id
        self.client_secret = client_secret
        self.user_id = user_id if user_id else None

    def get_access_token(self):
        """Получение токена в авито"""
        url_get_token = f"https://api.avito.ru/token/?grant_type=client_credentials&" \
                        f"client_id={self.client_id}&client_secret={self.client_secret}"
        r = requests.get(url=url_get_token)
        if 'error' not in r.text:
            access_token = r.json()['access_token']
            self.access_token = access_token
            return r.json()['access_token']
        return r.json()

    def get_last_report(self):
        """Последний отчет по выгрузке"""
        if not self.user_id:
            return {'errors': {'api_avito': 'Не указан user_id'}}
        headers = {"Authorization": f"Bearer {self.access_token}"}
        url_last_report = f"https://api.avito.ru/autoload/v1/accounts/{self.user_id}/reports/last_report/"
        r = requests.get(url=url_last_report, headers=headers)
        return r.json()

    def get_reports(self, per_page=10, page=1):
        """Получение всех отчетов по выгрузкам"""
        if not self.user_id:
            return {'errors': {'api_avito': 'Не указан user_id'}}
        url_reports = f"https://api.avito.ru/autoload/v1/accounts/{self.user_id}/reports/?per_page={per_page}&page={page}"
        headers = {"Authorization": f"Bearer {self.access_token}"}
        r = requests.get(url=url_reports, headers=headers)
        return r.json()
